-module(lesson).
-compile (export_all).

%% Main sections:
%% - functions
%% - dynamically typed
%% - atoms and variables
%% - pattern matching
%% - receives, sends, after
%% - spawn processes

%% Erlang is a functional language.
two_func () ->
     1 + 1.

%% It is also dynamically typed.
%% At least the compiler happens to warn us...
type_error () ->
    1 + "Hello".

%% As in most functional languages, 
%% recursion is used often.
factorial (N) ->
    if N > 1 -> N * factorial (N-1);
       N =< 1 -> N
    end.

%% Atoms (beginning with a lower-case letter)
%% act as tags. We are also doing pattern matching here.
prefix_name (male, Name) ->
    "Mr. " ++ Name;
prefix_name (female, Name) ->
    "Ms. " ++ Name.

%% We can also do simple IO
say_something () ->
    io:format ("What do I say?~n").


%% Messsages can be sent and received.
send_something (To) ->
    To ! "How are you?".
 
receive_it () ->
    receive {Message} ->
            io:format ("recv: ~s~n", [Message]),
            self () ! {Message ++ "!!!"},
            receive_it ()
    end.

%% Of course you need some different processes 
%% to have effective communication.
sender () ->
    Pid1 = spawn (?MODULE, waiter, [1]),
    Pid2 = spawn (?MODULE, waiter, [2]),
    Pid1 ! {self ()},
    Pid2 ! {self ()},
    receive {X, done} ->
            io:format ("Received ~B ~n", [X]),
            receive {Y, done} -> 
                    io:format ("Received ~B ~n", [Y])
            end
    end.

waiter (X) ->
    receive {From} -> timer:sleep (2000),
                      From ! {X, done}
    end.


%% You can maintain state by having a server pass
%% its state as an argument to a recursive function.
what_ive_seen (Stuff) ->
    receive 
        {More} when length (Stuff) < 3 ->
            what_ive_seen ([More | Stuff]);
        {_} -> 
            io:format ("I've seen enough!~n")
    end.
