import x10.util.Random;

/**
 *  A distributed random adder. It sums up distributed random numbers in parallel.
 */
class DistributedRandomAdder {
	/**
	 * Sum up the random numbers.
	 * To sum up the random numbers, the adder distributes the numbers across the available places. At each place, the adder uses a thread to compute the local sum as the sum of the numbers on the place. The adder then computes the global sum as the sum of all local sums.
	 * 
	 * @param arguments	the number of random numbers
	 */
	public static def main(arguments:Array[String](1)): Void {
		val numberOfElements = Int.parse(arguments(0)); // The number of random numbers.
		val randomNumberGenerator = new Random(); // The random number generator.
		val container = DistArray.make[Int](
			Dist.makeBlock(1..numberOfElements),
			([i]:Point) => randomNumberGenerator.nextInt(10)
		); // The container. It is a distributed array with random integer numbers.
		
		// Print the container.
		Console.OUT.print("container:");
		for (place in container.dist.places()) {
			at (place) {
				for (index in container.dist | here) {
					Console.OUT.print(" " + container(index));
				}
			}
		}
		Console.OUT.println();

		// Compute the local sums.
		val localSums = DistArray.make[Int](
			Dist.makeUnique(),
			(([i]:Point)=>0)
		); // The local sums. It contains a sum for each place.
		finish {
			for (place in container.dist.places()) {
				async at (place) {
					for (index in container.dist | here) {
						localSums(place.id) += container(index);
					}
				}
			}
		}

		// Sum up the local sums to get the global sum.
		var globalSum:Int = 0; // The global sum.
		for (place in localSums.dist.places()) {
			globalSum += (at (place) localSums(place.id));
		}
		
		// Print the global sum.
		Console.OUT.println("global sum: " + globalSum);
	}
}