import x10.util.Random;
import x10.lang.Math;

/**
 *  A parallel random adder. It sums up random numbers in parallel.
 */
class ParallelRandomAdder {
	/**
	 * Sum up the random numbers.
	 * To sum up the random numbers, the partitiones the number. For each partition, the adder uses a thread to compute the local sum as the sum of the numbers in that partition. The adder then computes the global sum as the sum of all local sums.
	 * 
	 * @param arguments	the number of random numbers and the number of partitions
	 */
	public static def main(arguments:Array[String](1)): Void {
		val numberOfElements = Int.parse(arguments(0)); // The number of random numbers.
		val numberOfPartitions = Int.parse(arguments(1)); // The number of partitions.
		val randomNumberGenerator = new Random(); // The random number generator.
		val container = new Array[Int](
			1..numberOfElements,
			([i]:Point) => randomNumberGenerator.nextInt(10)
		); // The container. It is an array with random integer numbers.
		
		// Print the container.
		Console.OUT.print("container:");
		for ([index] in 1..numberOfElements) {
			Console.OUT.print(" " + container(index));
		}
		Console.OUT.println();

		// Compute the local sums and the global sum.
		var globalSum:Int = 0; // The global sum.
		val localSums = new Array[Int](1..numberOfPartitions, ([i]:Point) => 0); // The local sums. It contains a sum for each partition.
		val maximumPartitionSize = (numberOfElements / numberOfPartitions) + (numberOfElements % numberOfPartitions > 0 ? 1 : 0); // The maximum size of a parition.
		finish {
			for ([partitionNumber] in 1..numberOfPartitions) {		
				async {
					for ([index] in (((partitionNumber - 1) * maximumPartitionSize) + 1)..(Math.min(partitionNumber * maximumPartitionSize, numberOfElements))) {
						localSums(partitionNumber) += container(index);
						
					}
					atomic globalSum += localSums(partitionNumber);
				}
			}
		}

		// Print the global sum.
		Console.OUT.println("global sum: " + globalSum);
	}
}